@@ -1,34 +1,32 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-"""
-helper class for making a SAT frontend
-Copyright (C) 2009, 2010, 2011, 2012, 2013  Jérôme Poisson (goffi@goffi.org)
+# helper class for making a SAT frontend
+# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from sat.core.i18n import _
+import sys
 from logging import debug, info, warning, error
 from sat.tools.jid  import JID
-from sat_frontends.bridge.DBus import DBusBridgeFrontend,BridgeExceptionNoService
+from sat_frontends.bridge.DBus import DBusBridgeFrontend
+from sat.core.exceptions import BridgeExceptionNoService, BridgeInitError
 from sat_frontends.quick_frontend.quick_utils import escapePrivate, unescapePrivate
 from optparse import OptionParser
 
-import sat_frontends.quick_frontend.constants
-
-import gettext
-gettext.install('sat_frontend', "../i18n", unicode=True)
+from sat_frontends.quick_frontend.constants import Const
 
 class QuickApp(object):
     """This class contain the main methods needed for the frontend"""
@@ -37,13 +35,15 @@ class QuickApp(object):
         self.profiles = {}
         self.single_profile = single_profile
         self.check_options()
-        
+
         ## bridge ##
         try:
             self.bridge=DBusBridgeFrontend()
         except BridgeExceptionNoService:
             print(_(u"Can't connect to SàT backend, are you sure it's launched ?"))
-            import sys
+            sys.exit(1)
+        except BridgeInitError:
+            print(_(u"Can't init bridge"))
             sys.exit(1)
         self.bridge.register("connected", self.connected)
         self.bridge.register("disconnected", self.disconnected)
@@ -81,11 +81,12 @@ class QuickApp(object):
         self.bridge.register("quizGameAnswerResult", self.quizGameAnswerResult, "plugin")
         self.bridge.register("quizGameTimerExpired", self.quizGameTimerExpired, "plugin")
         self.bridge.register("quizGameTimerRestarted", self.quizGameTimerRestarted, "plugin")
-        
+        self.bridge.register("chatStateReceived", self.chatStateReceived, "plugin")
+
         self.current_action_ids = set()
         self.current_action_ids_cb = {}
         self.media_dir = self.bridge.getConfig('','media_dir')
-    
+
     def check_profile(self, profile):
         """Tell if the profile is currently followed by the application"""
         return profile in self.profiles.keys()
@@ -93,7 +94,7 @@ class QuickApp(object):
     def postInit(self):
         """Must be called after initialization is done, do all automatic task (auto plug profile)"""
         if self.options.profile:
-            if not self.bridge.getProfileName(self.options.profile): 
+            if not self.bridge.getProfileName(self.options.profile):
                 error(_("Trying to plug an unknown profile (%s)" % self.options.profile))
             else:
                 self.plug_profile(self.options.profile)
@@ -116,7 +117,7 @@ class QuickApp(object):
 
     def _getParamError(self, ignore):
         error(_("Can't get profile parameter"))
-    
+
     def plug_profile(self, profile_key='@DEFAULT@'):
         """Tell application which profile must be used"""
         if self.single_profile and self.profiles:
@@ -132,11 +133,11 @@ class QuickApp(object):
         self.profiles[profile]={}
         if self.single_profile:
             self.profile = profile
-        
+
         ###now we get the essential params###
         self.bridge.asyncGetParamA("JabberID","Connection", profile_key=profile,
                                    callback=lambda _jid: self.plug_profile_2(_jid, profile), errback=self._getParamError)
-        
+
     def plug_profile_2(self, _jid, profile):
         self.profiles[profile]['whoami'] = JID(_jid)
         self.bridge.asyncGetParamA("autoconnect","Connection", profile_key=profile,
@@ -152,7 +153,7 @@ class QuickApp(object):
             self.bridge.asyncConnect(profile, callback=lambda: self.plug_profile_5(watched, autoconnect, profile), errback=lambda ignore: error(_('Error during autoconnection')))
         else:
             self.plug_profile_5(watched, autoconnect, profile)
-    
+
     def plug_profile_5(self, watched, autoconnect, profile):
         self.profiles[profile]['watched'] = watched.split() #TODO: put this in a plugin
 
@@ -198,7 +199,7 @@ class QuickApp(object):
             for confirm_id, confirm_type, data in self.bridge.getWaitingConf(profile):
                 self.askConfirmation(confirm_id, confirm_type, data, profile)
 
-        
+
 
     def unplug_profile(self, profile):
         """Tell the application to not follow anymore the profile"""
@@ -224,9 +225,9 @@ class QuickApp(object):
         debug(_("Disconnected"))
         self.contact_list.clearContacts()
         self.setStatusOnline(False)
-    
+
     def connectionError(self, error_type, profile):
-        """called when something goest wrong with the connection"""
+        """called when something goes wrong with the connection"""
         if not self.check_profile(profile):
             return
         debug(_("Connection Error"))
@@ -250,15 +251,15 @@ class QuickApp(object):
             return
         from_jid = JID(from_jid_s)
         to_jid = JID(to_jid_s)
-        
-        from_me = from_jid.short == self.profiles[profile]['whoami'].short
+
+        from_me = from_jid.bare == self.profiles[profile]['whoami'].bare
         win = to_jid if from_me else from_jid
-        
+
         if _type != "groupchat" and self.contact_list.getSpecial(win) == "MUC":
             #we have a private message in a MUC room
             #XXX: normaly we use bare jid as key, here we need the full jid
             #     so we cheat by replacing the "/" before the resource by
-            #     a "@", so the jid is invalid, 
+            #     a "@", so the jid is invalid,
             new_jid = escapePrivate(win)
             if from_me:
                 to_jid = new_jid
@@ -268,22 +269,26 @@ class QuickApp(object):
                 self.contact_list.add(new_jid)
 
         self.newMessage(from_jid, to_jid, msg, _type, extra, profile)
-    
+
     def newMessage(self, from_jid, to_jid, msg, _type, extra, profile):
-        from_me = from_jid.short == self.profiles[profile]['whoami'].short
+        from_me = from_jid.bare == self.profiles[profile]['whoami'].bare
         win = to_jid if from_me else from_jid
-        
+
         self.current_action_ids = set()
         self.current_action_ids_cb = {}
 
         timestamp = extra.get('archive')
-        self.chat_wins[win.short].printMessage(from_jid, msg, profile, float(timestamp) if timestamp else '')
+        self.chat_wins[win.bare].printMessage(from_jid, msg, profile, float(timestamp) if timestamp else '')
 
-    def sendMessage(self, to_jid, message, subject='', mess_type="auto", profile_key="@DEFAULT@"):
-        if to_jid.startswith(const_PRIVATE_PREFIX):
+    def sendMessage(self, to_jid, message, subject='', mess_type="auto", extra={}, callback=None, errback=None, profile_key="@NONE@"):
+        if to_jid.startswith(Const.PRIVATE_PREFIX):
             to_jid = unescapePrivate(to_jid)
             mess_type = "chat"
-        self.bridge.sendMessage(to_jid, message, subject, mess_type, profile_key)
+        if callback is None:
+            callback = lambda: None
+        if errback is None:
+            errback = lambda failure: self.showDialog(unicode(failure), _(u"sendMessage Error"), "error")
+        self.bridge.sendMessage(to_jid, message, subject, mess_type, extra, profile_key, callback=callback, errback=errback)
 
     def newAlert(self, msg, title, alert_type, profile):
         if not self.check_profile(profile):
@@ -291,39 +296,39 @@ class QuickApp(object):
         assert alert_type in ['INFO','ERROR']
         self.showDialog(unicode(msg),unicode(title),alert_type.lower())
 
-    
-    def setStatusOnline(self, online=True):
-        pass
+    def setStatusOnline(self, online=True, show="", statuses={}):
+        raise NotImplementedError
 
     def presenceUpdate(self, jabber_id, show, priority, statuses, profile):
         if not self.check_profile(profile):
             return
-        debug (_("presence update for %(jid)s (show=%(show)s, priority=%(priority)s, statuses=%(statuses)s) [profile:%(profile)s]") % {'jid':jabber_id, 'show':show, 'priority':priority, 'statuses':statuses, 'profile':profile});
-        from_jid=JID(jabber_id)
-        debug ("from_jid.short=%(from_jid)s whoami.short=%(whoami)s" % {'from_jid':from_jid.short, 'whoami':self.profiles[profile]['whoami'].short})
 
-        if from_jid.short==self.profiles[profile]['whoami'].short:
-            if not type:
-                self.setStatusOnline(True)
-            elif type=="unavailable":
+        debug(_("presence update for %(jid)s (show=%(show)s, priority=%(priority)s, statuses=%(statuses)s) [profile:%(profile)s]")
+              % {'jid': jabber_id, 'show': show, 'priority': priority, 'statuses': statuses, 'profile': profile})
+        from_jid = JID(jabber_id)
+
+        if from_jid == self.profiles[profile]['whoami']:
+            if show == "unavailable":
                 self.setStatusOnline(False)
+            else:
+                self.setStatusOnline(True, show, statuses)
             return
 
-        self.contact_list.updatePresence(from_jid, show, priority, statuses) 
-            
+        self.contact_list.updatePresence(from_jid, show, priority, statuses)
+
         if show != 'unavailable':
-            
+
             #FIXME: must be moved in a plugin
-            if from_jid.short in self.profiles[profile]['watched'] and not from_jid.short in self.profiles[profile]['onlineContact']:
-                self.showAlert(_("Watched jid [%s] is connected !") % from_jid.short)
+            if from_jid.bare in self.profiles[profile]['watched'] and not from_jid.bare in self.profiles[profile]['onlineContact']:
+                self.showAlert(_("Watched jid [%s] is connected !") % from_jid.bare)
+
+            self.profiles[profile]['onlineContact'].add(from_jid)  # FIXME onlineContact is useless with CM, must be removed
 
-            self.profiles[profile]['onlineContact'].add(from_jid)  #FIXME onlineContact is useless with CM, must be removed
-           
             #TODO: vcard data (avatar)
 
-        if show=="unavailable" and from_jid in self.profiles[profile]['onlineContact']:
+        if show == "unavailable" and from_jid in self.profiles[profile]['onlineContact']:
             self.profiles[profile]['onlineContact'].remove(from_jid)
-    
+
     def roomJoined(self, room_jid, room_nicks, user_nick, profile):
         """Called when a MUC room is joined"""
         if not self.check_profile(profile):
@@ -333,7 +338,7 @@ class QuickApp(object):
         self.chat_wins[room_jid].setType("group")
         self.chat_wins[room_jid].id = room_jid
         self.chat_wins[room_jid].setPresents(list(set([user_nick]+room_nicks)))
-        self.contact_list.setSpecial(JID(room_jid), "MUC")
+        self.contact_list.setSpecial(JID(room_jid), "MUC", show=True)
 
     def roomLeft(self, room_jid_s, profile):
         """Called when a MUC room is left"""
@@ -374,7 +379,7 @@ class QuickApp(object):
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].setSubject(subject)
             debug (_("new subject for room [%(room_jid)s]: %(subject)s") % {'room_jid':room_jid, "subject":subject})
-    
+
     def tarotGameStarted(self, room_jid, referee, players, profile):
         if not self.check_profile(profile):
             return
@@ -382,7 +387,7 @@ class QuickApp(object):
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].startGame("Tarot", referee, players)
             debug (_("new Tarot game started by [%(referee)s] in room [%(room_jid)s] with %(players)s") % {'referee':referee, 'room_jid':room_jid, 'players':[str(player) for player in players]})
-       
+
     def tarotGameNew(self, room_jid, hand, profile):
         if not self.check_profile(profile):
             return
@@ -411,8 +416,8 @@ class QuickApp(object):
         debug (_("My turn to play"))
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].getGame("Tarot").myTurn()
-    
-    def tarotScore(self, room_jid, xml_data, winners, loosers, profile): 
+
+    def tarotScore(self, room_jid, xml_data, winners, loosers, profile):
         """Called when the game is finished and the score are updated"""
         if not self.check_profile(profile):
             return
@@ -426,14 +431,14 @@ class QuickApp(object):
         debug (_("Card(s) played (%(player)s): %(cards)s") % {"player":player, "cards":cards})
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].getGame("Tarot").cardsPlayed(player, cards)
-   
+
     def tarotInvalidCards(self, room_jid, phase, played_cards, invalid_cards, profile):
         if not self.check_profile(profile):
             return
         debug (_("Cards played are not valid: %s") % invalid_cards)
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].getGame("Tarot").invalidCards(phase, played_cards, invalid_cards)
-  
+
     def quizGameStarted(self, room_jid, referee, players, profile):
         if not self.check_profile(profile):
             return
@@ -441,7 +446,7 @@ class QuickApp(object):
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].startGame("Quiz", referee, players)
             debug (_("new Quiz game started by [%(referee)s] in room [%(room_jid)s] with %(players)s") % {'referee':referee, 'room_jid':room_jid, 'players':[str(player) for player in players]})
-       
+
     def quizGameNew(self, room_jid, data, profile):
         if not self.check_profile(profile):
             return
@@ -477,7 +482,7 @@ class QuickApp(object):
             return
         if self.chat_wins.has_key(room_jid):
             self.chat_wins[room_jid].getGame("Quiz").quizGameAnswerResult(player, good_answer, score)
-    
+
     def quizGameTimerExpired(self, room_jid, profile):
         """Called when nobody answered the question in time"""
         if not self.check_profile(profile):
@@ -495,9 +500,9 @@ class QuickApp(object):
     def _subscribe_cb(self, answer, data):
         entity, profile = data
         if answer:
-            self.bridge.subscription("subscribed", entity.short, profile_key = profile)
+            self.bridge.subscription("subscribed", entity.bare, profile_key = profile)
         else:
-            self.bridge.subscription("unsubscribed", entity.short, profile_key = profile)
+            self.bridge.subscription("unsubscribed", entity.bare, profile_key = profile)
 
     def subscribe(self, type, raw_jid, profile):
         """Called when a subsciption management signal is received"""
@@ -506,20 +511,20 @@ class QuickApp(object):
         entity = JID(raw_jid)
         if type=="subscribed":
             # this is a subscription confirmation, we just have to inform user
-            self.showDialog(_("The contact %s has accepted your subscription") % entity.short, _('Subscription confirmation'))
+            self.showDialog(_("The contact %s has accepted your subscription") % entity.bare, _('Subscription confirmation'))
         elif type=="unsubscribed":
             # this is a subscription refusal, we just have to inform user
-            self.showDialog(_("The contact %s has refused your subscription") % entity.short, _('Subscription refusal'), 'error')
+            self.showDialog(_("The contact %s has refused your subscription") % entity.bare, _('Subscription refusal'), 'error')
         elif type=="subscribe":
             # this is a subscriptionn request, we have to ask for user confirmation
-            answer = self.showDialog(_("The contact %s wants to subscribe to your presence.\nDo you accept ?") % entity.short, _('Subscription confirmation'), 'yes/no', answer_cb = self._subscribe_cb, answer_data=(entity, profile))
+            answer = self.showDialog(_("The contact %s wants to subscribe to your presence.\nDo you accept ?") % entity.bare, _('Subscription confirmation'), 'yes/no', answer_cb = self._subscribe_cb, answer_data=(entity, profile))
 
     def showDialog(self, message, title, type="info", answer_cb = None):
         raise NotImplementedError
-    
+
     def showAlert(self, message):
         pass  #FIXME
-    
+
     def paramUpdate(self, name, value, namespace, profile):
         if not self.check_profile(profile):
             return
@@ -536,7 +541,7 @@ class QuickApp(object):
         target = JID(jid)
         self.contact_list.remove(target)
         try:
-            self.profiles[profile]['onlineContact'].remove(target.short)
+            self.profiles[profile]['onlineContact'].remove(target.bare)
         except KeyError:
             pass
 
@@ -556,7 +561,7 @@ class QuickApp(object):
 
     def askConfirmation(self, confirm_id, confirm_type, data, profile):
         raise NotImplementedError
-    
+
     def actionResult(self, type, id, data):
         raise NotImplementedError
 
@@ -570,3 +575,4 @@ class QuickApp(object):
                     self.bridge.disconnect(self.profile)
         except:
             pass
+
