@@ -1,24 +1,21 @@
 #!/usr/bin/python
 #-*- coding: utf-8 -*-
 
-"""
-SAT: a jabber client
-Copyright (C) 2009, 2010, 2011, 2012, 2013  Jérôme Poisson (goffi@goffi.org)
+# SAT: a jabber client
+# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
-
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
 
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from bridge import Bridge
 import dbus
@@ -27,43 +24,56 @@ import dbus.mainloop.glib
 import inspect
 from logging import debug, info, error
 from twisted.internet.defer import Deferred
+from sat.core.exceptions import BridgeInitError
 
-const_INT_PREFIX = "org.goffi.SAT"  #Interface prefix
-const_ERROR_PREFIX = const_INT_PREFIX+".error"
+const_INT_PREFIX = "org.goffi.SAT"  # Interface prefix
+const_ERROR_PREFIX = const_INT_PREFIX + ".error"
 const_OBJ_PATH = '/org/goffi/SAT/bridge'
 const_CORE_SUFFIX = ".core"
 const_PLUGIN_SUFFIX = ".plugin"
 
+
 class ParseError(Exception):
     pass
 
+
 class MethodNotRegistered(dbus.DBusException):
     _dbus_error_name = const_ERROR_PREFIX + ".MethodNotRegistered"
 
+
 class InternalError(dbus.DBusException):
     _dbus_error_name = const_ERROR_PREFIX + ".InternalError"
 
+
 class AsyncNotDeferred(dbus.DBusException):
     _dbus_error_name = const_ERROR_PREFIX + ".AsyncNotDeferred"
 
+
 class DeferredNotAsync(dbus.DBusException):
     _dbus_error_name = const_ERROR_PREFIX + ".DeferredNotAsync"
 
+
 class GenericException(dbus.DBusException):
     def __init__(self, twisted_error):
-        super(GenericException,self).__init__()
-        mess = twisted_error.getErrorMessage()
-        self._dbus_error_name = const_ERROR_PREFIX+"."+ (mess or str(twisted_error.__class__))
+        super(GenericException, self).__init__()
+        try:
+            error_name = str(twisted_error.value().__class__.__name__)
+            # twisted_error.value is a class
+        except TypeError:
+            # twisted_error.value is an instance
+            error_name = str(twisted_error.value.__class__.__name__)
+        self._dbus_error_name = const_ERROR_PREFIX + "." + error_name
+
 
 class DbusObject(dbus.service.Object):
 
     def __init__(self, bus, path):
         dbus.service.Object.__init__(self, bus, path)
         debug("Init DbusObject...")
-        self.cb={}
+        self.cb = {}
 
     def register(self, name, cb):
-        self.cb[name]=cb
+        self.cb[name] = cb
 
     def _callback(self, name, *args, **kwargs):
         """call the callback if it exists, raise an exception else
@@ -86,17 +96,16 @@ class DbusObject(dbus.service.Object):
             if not isinstance(result, Deferred):
                 error("Asynchronous method [%s] does not return a Deferred." % name)
                 raise AsyncNotDeferred
-            result.addCallback(lambda result: callback() if result==None else callback(result))
-            result.addErrback(lambda err:errback(GenericException(err)))
+            result.addCallback(lambda result: callback() if result is None else callback(result))
+            result.addErrback(lambda err: errback(GenericException(err)))
         else:
             if isinstance(result, Deferred):
                 error("Synchronous method [%s] return a Deferred." % name)
                 raise DeferredNotAsync
             return result
+    ### signals ###
 
-    ### signals ###    
-
-    @dbus.service.signal(const_INT_PREFIX+const_PLUGIN_SUFFIX,
+    @dbus.service.signal(const_INT_PREFIX + const_PLUGIN_SUFFIX,
                          signature='')
     def dummySignal(self):
         #FIXME: workaround for addSignal (doesn't work if one method doensn't
@@ -174,9 +183,8 @@ class DbusObject(dbus.service.Object):
     def subscribe(self, sub_type, entity_jid, profile):
         pass
 
+    ### methods ###
 
-    ### methods ###    
-    
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='',
                          async_callbacks=None)
@@ -196,16 +204,10 @@ class DbusObject(dbus.service.Object):
         return self._callback("asyncCreateProfile", unicode(profile), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssss', out_signature='s',
+                         in_signature='sssis', out_signature='s',
                          async_callbacks=('callback', 'errback'))
-    def asyncGetParamA(self, name, category, attribute="value", profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("asyncGetParamA", unicode(name), unicode(category), unicode(attribute), unicode(profile_key), callback=callback, errback=errback)
-
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssss', out_signature='s',
-                         async_callbacks=None)
-    def callMenu(self, category, name, menu_type, profile_key):
-        return self._callback("callMenu", unicode(category), unicode(name), unicode(menu_type), unicode(profile_key))
+    def asyncGetParamA(self, name, category, attribute="value", security_limit=-1, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("asyncGetParamA", unicode(name), unicode(category), unicode(attribute), security_limit, unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sba{ss}s', out_signature='',
@@ -268,7 +270,7 @@ class DbusObject(dbus.service.Object):
         return self._callback("getEntityData", unicode(jid), keys, unicode(profile))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssibs', out_signature='a(dssss)',
+                         in_signature='ssibs', out_signature='a(dssssa{ss})',
                          async_callbacks=('callback', 'errback'))
     def getHistory(self, from_jid, to_jid, limit, between=True, profile="@NONE@", callback=None, errback=None):
         return self._callback("getHistory", unicode(from_jid), unicode(to_jid), limit, between, unicode(profile), callback=callback, errback=errback)
@@ -280,16 +282,16 @@ class DbusObject(dbus.service.Object):
         return self._callback("getLastResource", unicode(contact_jid), unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='sss', out_signature='s',
+                         in_signature='ss', out_signature='s',
                          async_callbacks=None)
-    def getMenuHelp(self, category, name, menu_type):
-        return self._callback("getMenuHelp", unicode(category), unicode(name), unicode(menu_type))
+    def getMenuHelp(self, menu_id, language):
+        return self._callback("getMenuHelp", unicode(menu_id), unicode(language))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='', out_signature='a(sss)',
+                         in_signature='si', out_signature='a(ssasas)',
                          async_callbacks=None)
-    def getMenus(self, ):
-        return self._callback("getMenus", )
+    def getMenus(self, language, security_limit):
+        return self._callback("getMenus", unicode(language), security_limit)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ssss', out_signature='s',
@@ -298,10 +300,10 @@ class DbusObject(dbus.service.Object):
         return self._callback("getParamA", unicode(name), unicode(category), unicode(attribute), unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='s',
+                         in_signature='iss', out_signature='s',
                          async_callbacks=('callback', 'errback'))
-    def getParams(self, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("getParams", unicode(profile_key), callback=callback, errback=errback)
+    def getParams(self, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("getParams", security_limit, unicode(app), unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='', out_signature='as',
@@ -310,16 +312,16 @@ class DbusObject(dbus.service.Object):
         return self._callback("getParamsCategories", )
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ss', out_signature='s',
+                         in_signature='siss', out_signature='s',
                          async_callbacks=('callback', 'errback'))
-    def getParamsForCategory(self, category, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("getParamsForCategory", unicode(category), unicode(profile_key), callback=callback, errback=errback)
+    def getParamsForCategory(self, category, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("getParamsForCategory", unicode(category), security_limit, unicode(app), unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='s', out_signature='s',
+                         in_signature='iss', out_signature='s',
                          async_callbacks=('callback', 'errback'))
-    def getParamsUI(self, profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("getParamsUI", unicode(profile_key), callback=callback, errback=errback)
+    def getParamsUI(self, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("getParamsUI", security_limit, unicode(app), unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='a{sa{s(sia{ss})}}',
@@ -370,10 +372,16 @@ class DbusObject(dbus.service.Object):
         return self._callback("isConnected", unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='sa{ss}s', out_signature='s',
+                         in_signature='sa{ss}s', out_signature='a{ss}',
+                         async_callbacks=('callback', 'errback'))
+    def launchAction(self, callback_id, data, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("launchAction", unicode(callback_id), data, unicode(profile_key), callback=callback, errback=errback)
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='sis', out_signature='',
                          async_callbacks=None)
-    def launchAction(self, action_type, data, profile_key="@DEFAULT@"):
-        return self._callback("launchAction", unicode(action_type), data, unicode(profile_key))
+    def paramsRegisterApp(self, xml, security_limit=-1, app=''):
+        return self._callback("paramsRegisterApp", unicode(xml), security_limit, unicode(app))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ssssi', out_signature='s',
@@ -382,16 +390,16 @@ class DbusObject(dbus.service.Object):
         return self._callback("registerNewAccount", unicode(login), unicode(password), unicode(email), unicode(host), port)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='sssss', out_signature='',
-                         async_callbacks=None)
-    def sendMessage(self, to_jid, message, subject='', mess_type="auto", profile_key="@DEFAULT@"):
-        return self._callback("sendMessage", unicode(to_jid), unicode(message), unicode(subject), unicode(mess_type), unicode(profile_key))
+                         in_signature='ssssa{ss}s', out_signature='',
+                         async_callbacks=('callback', 'errback'))
+    def sendMessage(self, to_jid, message, subject='', mess_type="auto", extra={}, profile_key="@NONE@", callback=None, errback=None):
+        return self._callback("sendMessage", unicode(to_jid), unicode(message), unicode(subject), unicode(mess_type), extra, unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssss', out_signature='',
+                         in_signature='sssis', out_signature='',
                          async_callbacks=None)
-    def setParam(self, name, value, category, profile_key="@DEFAULT@"):
-        return self._callback("setParam", unicode(name), unicode(value), unicode(category), unicode(profile_key))
+    def setParam(self, name, value, category, security_limit=-1, profile_key="@DEFAULT@"):
+        return self._callback("setParam", unicode(name), unicode(value), unicode(category), security_limit, unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ssia{ss}s', out_signature='',
@@ -411,41 +419,40 @@ class DbusObject(dbus.service.Object):
     def updateContact(self, entity_jid, name, groups, profile_key="@DEFAULT@"):
         return self._callback("updateContact", unicode(entity_jid), unicode(name), groups, unicode(profile_key))
 
-    
     def __attributes(self, in_sign):
         """Return arguments to user given a in_sign
         @param in_sign: in_sign in the short form (using s,a,i,b etc)
-        @return: list of arguments that correspond to a in_sign (e.g.: "sss" return "arg1, arg2, arg3")""" 
-        i=0
-        idx=0
-        attr=[]
-        while i<len(in_sign):
-            if in_sign[i] not in ['b','y','n','i','x','q','u','t','d','s','a']:
+        @return: list of arguments that correspond to a in_sign (e.g.: "sss" return "arg1, arg2, arg3")"""
+        i = 0
+        idx = 0
+        attr = []
+        while i < len(in_sign):
+            if in_sign[i] not in ['b', 'y', 'n', 'i', 'x', 'q', 'u', 't', 'd', 's', 'a']:
                 raise ParseError("Unmanaged attribute type [%c]" % in_sign[i])
 
             attr.append("arg_%i" % idx)
-            idx+=1
+            idx += 1
 
             if in_sign[i] == 'a':
-                i+=1
-                if in_sign[i]!='{' and in_sign[i]!='(': #FIXME: must manage tuples out of arrays
-                    i+=1
-                    continue #we have a simple type for the array
+                i += 1
+                if in_sign[i] != '{' and in_sign[i] != '(':  # FIXME: must manage tuples out of arrays
+                    i += 1
+                    continue  # we have a simple type for the array
                 opening_car = in_sign[i]
-                assert(opening_car in ['{','('])
+                assert(opening_car in ['{', '('])
                 closing_car = '}' if opening_car == '{' else ')'
                 opening_count = 1
-                while (True): #we have a dict or a list of tuples
-                    i+=1
-                    if i>=len(in_sign):
+                while (True):  # we have a dict or a list of tuples
+                    i += 1
+                    if i >= len(in_sign):
                         raise ParseError("missing }")
                     if in_sign[i] == opening_car:
-                        opening_count+=1
+                        opening_count += 1
                     if in_sign[i] == closing_car:
-                        opening_count-=1
+                        opening_count -= 1
                         if opening_count == 0:
                             break
-            i+=1
+            i += 1
         return attr
 
     def addMethod(self, name, int_suffix, in_sign, out_sign, method, async=False):
@@ -454,58 +461,63 @@ class DbusObject(dbus.service.Object):
 
         _arguments = inspect_args.args
         _defaults = list(inspect_args.defaults or [])
-        
+
         if inspect.ismethod(method):
             #if we have a method, we don't want the first argument (usually 'self')
             del(_arguments[0])
 
         #first arguments are for the _callback method
-        arguments_callback = ', '.join([repr(name)] + ((_arguments + ['callback=callback','errback=errback']) if async else _arguments))
-        
+        arguments_callback = ', '.join([repr(name)] + ((_arguments + ['callback=callback', 'errback=errback']) if async else _arguments))
+
         if async:
-            _arguments.extend(['callback','errback'])
+            _arguments.extend(['callback', 'errback'])
             _defaults.extend([None, None])
-        
-        
+
         #now we create a second list with default values
-        for i in range(1, len(_defaults)+1):
+        for i in range(1, len(_defaults) + 1):
             _arguments[-i] = "%s = %s" % (_arguments[-i], repr(_defaults[-i]))
 
-        arguments_defaults =  ', '.join(_arguments)
+        arguments_defaults = ', '.join(_arguments)
 
-        code = compile ('def %(name)s (self,%(arguments_defaults)s): return self._callback(%(arguments_callback)s)' %
-                        {'name':name, 'arguments_defaults':arguments_defaults, 'arguments_callback':arguments_callback}, '<DBus bridge>','exec')
-        exec (code) #FIXME: to the same thing in a cleaner way, without compile/exec
+        code = compile('def %(name)s (self,%(arguments_defaults)s): return self._callback(%(arguments_callback)s)' %
+                       {'name': name, 'arguments_defaults': arguments_defaults, 'arguments_callback': arguments_callback}, '<DBus bridge>', 'exec')
+        exec (code)  # FIXME: to the same thing in a cleaner way, without compile/exec
         method = locals()[name]
         async_callbacks = ('callback', 'errback') if async else None
         setattr(DbusObject, name, dbus.service.method(
-            const_INT_PREFIX+int_suffix, in_signature=in_sign, out_signature=out_sign,
+            const_INT_PREFIX + int_suffix, in_signature=in_sign, out_signature=out_sign,
             async_callbacks=async_callbacks)(method))
         function = getattr(self, name)
         func_table = self._dbus_class_table[self.__class__.__module__ + '.' + self.__class__.__name__][function._dbus_interface]
-        func_table[function.__name__] = function #Needed for introspection
-    
+        func_table[function.__name__] = function  # Needed for introspection
+
     def addSignal(self, name, int_suffix, signature, doc={}):
         """Dynamically add a signal to Dbus Bridge"""
         attributes = ', '.join(self.__attributes(signature))
         #TODO: use doc parameter to name attributes
 
         #code = compile ('def '+name+' (self,'+attributes+'): debug ("'+name+' signal")', '<DBus bridge>','exec') #XXX: the debug is too annoying with xmllog
-        code = compile ('def '+name+' (self,'+attributes+'): pass', '<DBus bridge>','exec')
+        code = compile('def ' + name + ' (self,' + attributes + '): pass', '<DBus bridge>', 'exec')
         exec (code)
         signal = locals()[name]
         setattr(DbusObject, name, dbus.service.signal(
-            const_INT_PREFIX+int_suffix, signature=signature)(signal))
+            const_INT_PREFIX + int_suffix, signature=signature)(signal))
         function = getattr(self, name)
         func_table = self._dbus_class_table[self.__class__.__module__ + '.' + self.__class__.__name__][function._dbus_interface]
-        func_table[function.__name__] = function #Needed for introspection
+        func_table[function.__name__] = function  # Needed for introspection
+
 
 class DBusBridge(Bridge):
     def __init__(self):
         dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
         Bridge.__init__(self)
-        info ("Init DBus...")
-        self.session_bus = dbus.SessionBus()
+        info("Init DBus...")
+        try:
+            self.session_bus = dbus.SessionBus()
+        except dbus.DBusException as e:
+            if e._dbus_error_name == 'org.freedesktop.DBus.Error.NotSupported':
+                print u"D-Bus is not launched, please see README to see instructions on how to launch it"
+            raise BridgeInitError
         self.dbus_name = dbus.service.BusName(const_INT_PREFIX, self.session_bus)
         self.dbus_bridge = DbusObject(self.session_bus, const_OBJ_PATH)
 
@@ -551,7 +563,6 @@ class DBusBridge(Bridge):
     def subscribe(self, sub_type, entity_jid, profile):
         self.dbus_bridge.subscribe(sub_type, entity_jid, profile)
 
-
     def register(self, name, callback):
         debug("registering DBus bridge method [%s]", name)
         self.dbus_bridge.register(name, callback)
@@ -565,4 +576,4 @@ class DBusBridge(Bridge):
 
     def addSignal(self, name, int_suffix, signature, doc={}):
         self.dbus_bridge.addSignal(name, int_suffix, signature, doc)
-        setattr(DBusBridge, name, getattr(self.dbus_bridge, name))
+        setattr(DBusBridge, name, getattr(self.dbus_bridge, name))
\ No newline at end of file
