@@ -1,45 +1,49 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-"""
-SAT: a jabber client
-Copyright (C) 2009, 2010, 2011, 2012, 2013  Jérôme Poisson (goffi@goffi.org)
+# SAT: a jabber client
+# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from sat.core.i18n import _
 from twisted.internet import task, defer
 from twisted.words.protocols.jabber import jid, xmlstream
-from wokkel import client, disco, xmppim, generic, compat, delay
-from logging import debug, info, error
+from wokkel import client, disco, xmppim, generic, compat, delay, iwokkel
+from logging import debug, info, warning, error
 from sat.core import exceptions
 from calendar import timegm
+from zope.interface import implements
+try:
+    from twisted.words.protocols.xmlstream import XMPPHandler
+except ImportError:
+    from wokkel.subprotocols import XMPPHandler
 
 
 class SatXMPPClient(client.XMPPClient):
-    
+    implements(iwokkel.IDisco)
+
     def __init__(self, host_app, profile, user_jid, password, host=None, port=5222):
         client.XMPPClient.__init__(self, user_jid, password, host, port)
         self.factory.clientConnectionLost = self.connectionLost
-        self.__connected=False
+        self.__connected = False
         self.profile = profile
         self.host_app = host_app
         self.client_initialized = defer.Deferred()
         self.conn_deferred = defer.Deferred()
-        self._waiting_conf = {}  #callback called when a confirmation is received
-        self._progress_cb_map = {}  #callback called when a progress is requested (key = progress id)
-        
+        self._waiting_conf = {}  # callback called when a confirmation is received
+        self._progress_cb_map = {}  # callback called when a progress is requested (key = progress id)
 
     def getConnectionDeferred(self):
         """Return a deferred which fire when the client is connected"""
@@ -49,86 +53,106 @@ class SatXMPPClient(client.XMPPClient):
         if not self.host_app.trigger.point("XML Initialized", xmlstream, self.profile):
             return
         client.XMPPClient._authd(self, xmlstream)
-        self.__connected=True
-        info (_("********** [%s] CONNECTED **********") % self.profile)
+        self.__connected = True
+        info(_("********** [%s] CONNECTED **********") % self.profile)
         self.streamInitialized()
-        self.host_app.bridge.connected(self.profile) #we send the signal to the clients
-   
+        self.host_app.bridge.connected(self.profile)  # we send the signal to the clients
 
     def streamInitialized(self):
         """Called after _authd"""
-        debug (_("XML stream is initialized"))
-        self.keep_alife = task.LoopingCall(self.xmlstream.send, " ")  #Needed to avoid disconnection (specially with openfire)
+        debug(_("XML stream is initialized"))
+        self.keep_alife = task.LoopingCall(self.xmlstream.send, " ")  # Needed to avoid disconnection (specially with openfire)
         self.keep_alife.start(180)
-        
+
         self.disco = SatDiscoProtocol(self)
         self.disco.setHandlerParent(self)
         self.discoHandler = disco.DiscoHandler()
         self.discoHandler.setHandlerParent(self)
-        
+
         if not self.host_app.trigger.point("Disco Handled", self.profile):
             return
 
         self.roster.requestRoster()
-        
+
         self.presence.available()
-       
-        self.disco.requestInfo(jid.JID(self.jid.host)).addCallback(self.host_app.serverDisco, self.profile)  #FIXME: use these informations
-        
-        self.disco.requestItems(jid.JID(self.jid.host)).addCallback(self.host_app.serverDiscoItems, self.disco, self.profile, self.client_initialized)
+
+        jid_ = self.getHostJid()
+        self.disco.requestInfo(jid_).addCallback(self.host_app.serverDisco, jid_, self.profile)  # FIXME: use these informations
+
+        self.disco.requestItems(jid_).addCallback(self.host_app.serverDiscoItems, self.disco, jid_, self.profile, self.client_initialized)
         self.conn_deferred.callback(None)
 
     def initializationFailed(self, reason):
-        print ("initializationFailed: %s" % reason)
+        print "initializationFailed: %s" % reason
         self.host_app.bridge.connectionError("AUTH_ERROR", self.profile)
         try:
             client.XMPPClient.initializationFailed(self, reason)
         except:
-            #we already send an error signal, no need to raise an exception
+            # we already send an error signal, no need to raise an exception
             pass
         self.conn_deferred.errback()
 
     def isConnected(self):
         return self.__connected
-    
+
     def connectionLost(self, connector, unused_reason):
-        self.__connected=False
-        info (_("********** [%s] DISCONNECTED **********") % self.profile)
+        self.__connected = False
+        info(_("********** [%s] DISCONNECTED **********") % self.profile)
         try:
             self.keep_alife.stop()
         except AttributeError:
-            debug (_("No keep_alife"))
-        self.host_app.bridge.disconnected(self.profile) #we send the signal to the clients
-        self.host_app.purgeClient(self.profile) #and we remove references to this client
+            debug(_("No keep_alife"))
+        self.host_app.bridge.disconnected(self.profile)  # we send the signal to the clients
+        self.host_app.purgeClient(self.profile)  # and we remove references to this client
+
+    def getHostJid(self):
+        """@return: the jid of the local server"""
+        return jid.JID(self.jid.host)
 
 
 class SatMessageProtocol(xmppim.MessageProtocol):
-    
+
     def __init__(self, host):
         xmppim.MessageProtocol.__init__(self)
         self.host = host
 
     def onMessage(self, message):
-        debug (_(u"got message from: %s"), message["from"])
-        if not self.host.trigger.point("MessageReceived", message, profile=self.parent.profile):
+        debug(_(u"got message from: %s"), message["from"])
+        post_treat = defer.Deferred() # XXX: plugin can add their treatments to this deferred
+
+        if not self.host.trigger.point("MessageReceived", message, post_treat, profile=self.parent.profile):
             return
+
+        data = {"from": message['from'],
+                "to": message['to'],
+                "body": "",
+                "extra": {}}
+
         for e in message.elements():
             if e.name == "body":
-                mess_type = message['type'] if message.hasAttribute('type') else 'normal'
-                mess_body = e.children[0] if e.children else ""
-                try:
-                    _delay = delay.Delay.fromElement(filter(lambda elm: elm.name == 'delay', message.elements())[0])
-                    timestamp = timegm(_delay.stamp.utctimetuple())
-                    extra = {"archive": str(timestamp)}
-                    if mess_type != 'groupchat': #XXX: we don't save delayed messages in history for groupchats
-                        #TODO: add delayed messages to history if they aren't already in it
-                        self.host.memory.addToHistory(jid.JID(message["from"]), jid.JID(message["to"]), mess_body, mess_type, timestamp, profile=self.parent.profile)
-                except IndexError:
-                    extra = {}
-                    self.host.memory.addToHistory(jid.JID(message["from"]), jid.JID(message["to"]), mess_body, mess_type, profile=self.parent.profile)
-                self.host.bridge.newMessage(message["from"], mess_body, mess_type, message['to'], extra, profile=self.parent.profile)
+                data['body'] = e.children[0] if e.children else ""
                 break
-    
+
+        data['type'] = message['type'] if message.hasAttribute('type') else 'normal'
+
+        def after_treatments(data):
+            # set message body to empty string by default, and later
+            # also forward message without body (chat state notification...)
+            try:
+                _delay = delay.Delay.fromElement(filter(lambda elm: elm.name == 'delay', message.elements())[0])
+                timestamp = timegm(_delay.stamp.utctimetuple())
+                data['extra']['archive'] = str(timestamp)
+                if data['type'] != 'groupchat':  # XXX: we don't save delayed messages in history for groupchats
+                    #TODO: add delayed messages to history if they aren't already in it
+                    self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], timestamp, profile=self.parent.profile)
+            except IndexError:
+                self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], profile=self.parent.profile)
+            self.host.bridge.newMessage(data['from'], data['body'], data['type'], data['to'], data['extra'], profile=self.parent.profile)
+
+        post_treat.addCallback(after_treatments)
+        post_treat.callback(data)
+
+
 class SatRosterProtocol(xmppim.RosterClientProtocol):
 
     def __init__(self, host):
@@ -136,9 +160,9 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         self.host = host
         self.got_roster = defer.Deferred()
         #XXX: the two following dicts keep a local copy of the roster
-        self._groups = {} #map from groups to bare jids: key=group value=set of bare jids
-        self._jids = {} #map from bare jids to RosterItem: key=jid value=RosterItem
-    
+        self._groups = {}  # map from groups to bare jids: key=group value=set of bare jids
+        self._jids = {}  # map from bare jids to RosterItem: key=jid value=RosterItem
+
     def rosterCb(self, roster):
         for raw_jid, item in roster.iteritems():
             self.onRosterSet(item)
@@ -149,11 +173,13 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         d = self.getRoster().addCallback(self.rosterCb)
         d.chainDeferred(self.got_roster)
 
-    def removeItem(self, to):
-        """Remove a contact from roster list"""
-        xmppim.RosterClientProtocol.removeItem(self, to)
+    def removeItem(self, to_jid):
+        """Remove a contact from roster list
+        @param to_jid: a JID instance
+        """
+        xmppim.RosterClientProtocol.removeItem(self, to_jid)
         #TODO: check IQ result
-    
+
     #XXX: disabled (cf http://wokkel.ik.nu/ticket/56))
     #def addItem(self, to):
         #"""Add a contact to roster list"""
@@ -187,35 +213,35 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         if item.name:
             item_attr['name'] = item.name
         return item_attr
-    
+
     def onRosterSet(self, item):
         """Called when a new/update roster item is received"""
         #TODO: send a signal to frontends
         if not item.subscriptionTo and not item.subscriptionFrom and not item.ask:
             #XXX: current behaviour: we don't want contact in our roster list
-            #if there is no presence subscription
-            #may change in the future
+            # if there is no presence subscription
+            # may change in the future
             self.removeItem(item.jid)
             return
-        info (_("new contact in roster list: %s"), item.jid.full())
+        info(_("new contact in roster list: %s"), item.jid.full())
         #self.host.memory.addContact(item.jid, item_attr, item.groups, self.parent.profile)
-       
-        bare_jid = item.jid.userhost()
+
+        bare_jid = item.jid.userhostJID()
         self._jids[bare_jid] = item
         for group in item.groups:
-            self._groups.setdefault(group,set()).add(bare_jid)
+            self._groups.setdefault(group, set()).add(bare_jid)
         self.host.bridge.newContact(item.jid.full(), self.getAttributes(item), item.groups, self.parent.profile)
-    
+
     def onRosterRemove(self, entity):
         """Called when a roster removal event is received"""
         print _("removing %s from roster list") % entity.full()
-        bare_jid = entity.userhost()
+        bare_jid = entity.userhostJID()
 
-        #we first remove item from local cache (self._groups and self._jids)
+        # we first remove item from local cache (self._groups and self._jids)
         try:
             item = self._jids.pop(bare_jid)
         except KeyError:
-            log.warning("Received a roster remove event for an item not in cache")
+            warning("Received a roster remove event for an item not in cache")
             return
         for group in item.groups:
             try:
@@ -224,10 +250,10 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
                 if not jids_set:
                     del self._groups[group]
             except KeyError:
-                log.warning("there is not cache for the group [%(groups)s] of the removed roster item [%(jid)s]" %
-                            {"group": group, "jid": bare_jid})
-                                                                                                                            
-        #then we send the bridge signal
+                warning("there is not cache for the group [%(groups)s] of the removed roster item [%(jid)s]" %
+                        {"group": group, "jid": bare_jid})
+
+        # then we send the bridge signal
         self.host.bridge.contactDeleted(entity.userhost(), self.parent.profile)
 
     def getGroups(self):
@@ -238,7 +264,7 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         """Return RosterItem for a given jid
         @param jid: jid of the contact
         @return: RosterItem or None if contact is not in cache"""
-        return self._jids.get(jid.userhost(), None)
+        return self._jids.get(jid.userhostJID(), None)
 
     def getBareJids(self):
         """Return all bare jids (as unicode) of the roster"""
@@ -246,8 +272,8 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
 
     def isJidInRoster(self, entity_jid):
         """Return True if jid is in roster"""
-        return entity_jid.userhost() in self._jids
-    
+        return entity_jid.userhostJID() in self._jids
+
     def getItems(self):
         """Return all items of the roster"""
         return self._jids.values()
@@ -256,60 +282,68 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         try:
             return self._groups[group]
         except KeyError:
-            return  exceptions.UnknownGroupError
-        
+            raise exceptions.UnknownGroupError
+
 
 class SatPresenceProtocol(xmppim.PresenceClientProtocol):
 
     def __init__(self, host):
         xmppim.PresenceClientProtocol.__init__(self)
         self.host = host
-    
+
     def availableReceived(self, entity, show=None, statuses=None, priority=0):
-        debug (_("presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity':entity, 'show':show, 'statuses':statuses, 'priority':priority})
-        
+        debug(_("presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity': entity, 'show': show, 'statuses': statuses, 'priority': priority})
+
         if not statuses:
             statuses = {}
-        
-        if statuses.has_key(None):   #we only want string keys
+
+        if None in statuses:  # we only want string keys
             statuses["default"] = statuses[None]
             del statuses[None]
 
         self.host.memory.setPresenceStatus(entity, show or "",
-                int(priority), statuses, self.parent.profile)
+                                           int(priority), statuses,
+                                           self.parent.profile)
+
+        # now it's time to notify frontends
+        self.host.bridge.presenceUpdate(entity.full(), show or "",
+                                        int(priority), statuses,
+                                        self.parent.profile)
 
-        #now it's time to notify frontends
-        self.host.bridge.presenceUpdate(entity.full(),  show or "",
-                int(priority), statuses, self.parent.profile)
-    
     def unavailableReceived(self, entity, statuses=None):
-        debug (_("presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity':entity, 'statuses':statuses})
-        
+        debug(_("presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity': entity, 'statuses': statuses})
+
         if not statuses:
             statuses = {}
-        
-        if statuses.has_key(None):   #we only want string keys
+
+        if None in statuses:  # we only want string keys
             statuses["default"] = statuses[None]
             del statuses[None]
         self.host.memory.setPresenceStatus(entity, "unavailable", 0, statuses, self.parent.profile)
 
-        #now it's time to notify frontends
+        # now it's time to notify frontends
         self.host.bridge.presenceUpdate(entity.full(), "unavailable", 0, statuses, self.parent.profile)
-        
-    
-    def available(self, entity=None, show=None, statuses=None, priority=0):
-        if not statuses:
-            statuses = {}
-            # default for us is None for wokkel
-            # so we must temporarily switch to wokkel's convention...	
-            if 'default' in statuses:
-                statuses[None] = statuses['default']
 
-            xmppim.PresenceClientProtocol.available(self, entity, show, statuses, priority)
-            presence_elt = xmppim.AvailablePresence(entity, show, statuses, priority)
-            if not self.host.trigger.point("presence_available", presence_elt, self.parent):
-                return
-            self.send(presence_elt)
+    def available(self, entity=None, show=None, statuses=None, priority=None):
+        if priority is None:
+            try:
+                priority = int(self.host.memory.getParamA("Priority", "Connection", profile_key=self.parent.profile))
+            except ValueError:
+                priority = 0
+
+        if statuses is None:
+            statuses = {}
+
+        # default for us is None for wokkel
+        # so we must temporarily switch to wokkel's convention...
+        if 'default' in statuses:
+            statuses[None] = statuses['default']
+
+        xmppim.PresenceClientProtocol.available(self, entity, show, statuses, priority)
+        presence_elt = xmppim.AvailablePresence(entity, show, statuses, priority)
+        if not self.host.trigger.point("presence_available", presence_elt, self.parent):
+            return
+        self.send(presence_elt)
 
         # ... before switching back
         if None in statuses:
@@ -319,7 +353,7 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         xmppim.PresenceClientProtocol.subscribed(self, entity)
         self.host.memory.delWaitingSub(entity.userhost(), self.parent.profile)
         item = self.parent.roster.getItem(entity)
-        if not item or not item.subscriptionTo: #we automatically subscribe to 'to' presence
+        if not item or not item.subscriptionTo:  # we automatically subscribe to 'to' presence
             debug(_('sending automatic "from" subscription request'))
             self.subscribe(entity)
 
@@ -328,18 +362,18 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         self.host.memory.delWaitingSub(entity.userhost(), self.parent.profile)
 
     def subscribedReceived(self, entity):
-        debug (_("subscription approved for [%s]") % entity.userhost())
+        debug(_("subscription approved for [%s]") % entity.userhost())
         self.host.bridge.subscribe('subscribed', entity.userhost(), self.parent.profile)
 
     def unsubscribedReceived(self, entity):
-        debug (_("unsubscription confirmed for [%s]") % entity.userhost())
+        debug(_("unsubscription confirmed for [%s]") % entity.userhost())
         self.host.bridge.subscribe('unsubscribed', entity.userhost(), self.parent.profile)
 
     def subscribeReceived(self, entity):
-        debug (_("subscription request from [%s]") % entity.userhost())
+        debug(_("subscription request from [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
         if item and item.subscriptionTo:
-            #We automatically accept subscription if we are already subscribed to contact presence
+            # We automatically accept subscription if we are already subscribed to contact presence
             debug(_('sending automatic subscription acceptance'))
             self.subscribed(entity)
         else:
@@ -347,27 +381,30 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
             self.host.bridge.subscribe('subscribe', entity.userhost(), self.parent.profile)
 
     def unsubscribeReceived(self, entity):
-        debug (_("unsubscription asked for [%s]") % entity.userhost())
+        debug(_("unsubscription asked for [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
-        if item and item.subscriptionFrom: #we automatically remove contact
+        if item and item.subscriptionFrom:  # we automatically remove contact
             debug(_('automatic contact deletion'))
-            self.host.delContact(entity.userhost(), self.parent.profile)
+            self.host.delContact(entity, self.parent.profile)
         self.host.bridge.subscribe('unsubscribe', entity.userhost(), self.parent.profile)
 
+
 class SatDiscoProtocol(disco.DiscoClientProtocol):
     def __init__(self, host):
         disco.DiscoClientProtocol.__init__(self)
 
+
 class SatFallbackHandler(generic.FallbackHandler):
     def __init__(self, host):
         generic.FallbackHandler.__init__(self)
 
     def iqFallback(self, iq):
-        if iq.handled == True:
+        if iq.handled is True:
             return
-        debug (u"iqFallback: xml = [%s]" % (iq.toXml()))
+        debug(u"iqFallback: xml = [%s]" % (iq.toXml()))
         generic.FallbackHandler.iqFallback(self, iq)
 
+
 class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
 
     def __init__(self, host, jabber_host, user_login, user_pass, email, answer_id, profile):
@@ -379,11 +416,11 @@ class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
         self.user_email = email
         self.answer_id = answer_id
         self.profile = profile
-        print _("Registration asked for"),user_login, user_pass, jabber_host
-    
+        print _("Registration asked for"), user_login, user_pass, jabber_host
+
     def connectionMade(self):
         print "connectionMade"
-        
+
         self.xmlstream.namespace = "jabber:client"
         self.xmlstream.sendHeader()
 
@@ -400,25 +437,26 @@ class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
         reg = iq.send(self.jabber_host).addCallbacks(self.registrationAnswer, self.registrationFailure)
 
     def registrationAnswer(self, answer):
-        debug (_("registration answer: %s") % answer.toXml())
+        debug(_("registration answer: %s") % answer.toXml())
         answer_type = "SUCCESS"
-        answer_data={"message":_("Registration successfull")}
+        answer_data = {"message": _("Registration successfull")}
         self.host.bridge.actionResult(answer_type, self.answer_id, answer_data, self.profile)
         self.xmlstream.sendFooter()
-        
+
     def registrationFailure(self, failure):
-        info (_("Registration failure: %s") % str(failure.value))
+        info(_("Registration failure: %s") % str(failure.value))
         answer_type = "ERROR"
         answer_data = {}
         if failure.value.condition == 'conflict':
             answer_data['reason'] = 'conflict'
-            answer_data={"message":_("Username already exists, please choose an other one")}
+            answer_data = {"message": _("Username already exists, please choose an other one")}
         else:
             answer_data['reason'] = 'unknown'
-            answer_data={"message":_("Registration failed (%s)") % str(failure.value.condition)}
+            answer_data = {"message": _("Registration failed (%s)") % str(failure.value.condition)}
         self.host.bridge.actionResult(answer_type, self.answer_id, answer_data, self.profile)
         self.xmlstream.sendFooter()
 
+
 class SatVersionHandler(generic.VersionHandler):
 
     def getDiscoInfo(self, requestor, target, node):
@@ -429,3 +467,15 @@ class SatVersionHandler(generic.VersionHandler):
         # it reject our features (resulting in e.g. no notification on PEP)
         return generic.VersionHandler.getDiscoInfo(self, requestor, target, None)
 
+class SatIdentityHandler(XMPPHandler):
+    """ Manage disco Identity of SàT. Currently, we use "client/pc/Salut à Toi", but as
+    SàT is multi-frontends and can be used on mobile devices, as a bot, with a web frontend,
+    etc, we should implement a way to dynamically update identities through the bridge """
+    #TODO: dynamic identity update (see docstring). Note that a XMPP entity can have several identities
+    implements(iwokkel.IDisco)
+
+    def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
+        return [disco.DiscoIdentity(u"client", u"pc", u"Salut à Toi")]
+
+    def getDiscoItems(self, requestor, target, nodeIdentifier=''):
+        return []
