@@ -1,60 +1,60 @@
 #!/usr/bin/python
 #-*- coding: utf-8 -*-
-"""
-SAT plugin for managing xep-0065
 
-Copyright (C)
-2002, 2003, 2004   Dave Smith (dizzyd@jabber.org)
-2007, 2008         Fabio Forno (xmpp:ff@jabber.bluendo.com)
-2009, 2010, 2011, 2012, 2013  Jérôme Poisson (goffi@goffi.org)
+# SAT plugin for managing xep-0065
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# Copyright (C)
+# 2002, 2003, 2004   Dave Smith (dizzyd@jabber.org)
+# 2007, 2008         Fabio Forno (xmpp:ff@jabber.bluendo.com)
+# 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
 
---
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-This program is based on proxy65 (http://code.google.com/p/proxy65),
-originaly written by David Smith and modified by Fabio Forno.
-It is sublicensed under GPL v3 (or any later version) as allowed by the original
-license.
+# --
 
---
+# This program is based on proxy65 (http://code.google.com/p/proxy65),
+# originaly written by David Smith and modified by Fabio Forno.
+# It is sublicensed under AGPL v3 (or any later version) as allowed by the original
+# license.
 
-Here is a copy of the original license:
+# --
 
-Copyright (C) 
-2002-2004   Dave Smith (dizzyd@jabber.org)
-2007-2008   Fabio Forno (xmpp:ff@jabber.bluendo.com)
+# Here is a copy of the original license:
 
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
+# Copyright (C)
+# 2002-2004   Dave Smith (dizzyd@jabber.org)
+# 2007-2008   Fabio Forno (xmpp:ff@jabber.bluendo.com)
 
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
-"""
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
+
+from sat.core.i18n import _
 from logging import debug, info, warning, error
 from twisted.internet import protocol, reactor
 from twisted.internet import error as jab_error
@@ -78,30 +78,28 @@ from wokkel import disco, iwokkel
 IQ_SET = '/iq[@type="set"]'
 NS_BS = 'http://jabber.org/protocol/bytestreams'
 BS_REQUEST = IQ_SET + '/query[@xmlns="' + NS_BS + '"]'
-TIMEOUT = 60 #timeout for workflow
-
-
+TIMEOUT = 60  # timeout for workflow
 
 PLUGIN_INFO = {
-"name": "XEP 0065 Plugin",
-"import_name": "XEP-0065",
-"type": "XEP",
-"protocols": ["XEP-0065"],
-"main": "XEP_0065",
-"handler": "yes",
-"description": _("""Implementation of SOCKS5 Bytestreams""")
+    "name": "XEP 0065 Plugin",
+    "import_name": "XEP-0065",
+    "type": "XEP",
+    "protocols": ["XEP-0065"],
+    "main": "XEP_0065",
+    "handler": "yes",
+    "description": _("""Implementation of SOCKS5 Bytestreams""")
 }
 
 STATE_INITIAL = 0
-STATE_AUTH    = 1
+STATE_AUTH = 1
 STATE_REQUEST = 2
-STATE_READY   = 3
+STATE_READY = 3
 STATE_AUTH_USERPASS = 4
 STATE_TARGET_INITIAL = 5
-STATE_TARGET_AUTH    = 6
+STATE_TARGET_AUTH = 6
 STATE_TARGET_REQUEST = 7
-STATE_TARGET_READY   = 8
-STATE_LAST    = 9
+STATE_TARGET_READY = 8
+STATE_LAST = 9
 
 STATE_CONNECT_PENDING = STATE_LAST + 1
 
@@ -139,15 +137,14 @@ def calculateHash(from_jid, to_jid, sid):
     return hashlib.sha1((sid + from_jid.full() + to_jid.full()).encode('utf-8')).hexdigest()
 
 
-
 class SOCKSv5(protocol.Protocol, FileSender):
     def __init__(self):
         debug(_("Protocol init"))
         self.state = STATE_INITIAL
         self.buf = ""
-        self.supportedAuthMechs = [ AUTHMECH_ANON ]
-        self.supportedAddrs = [ ADDR_DOMAINNAME ]
-        self.enabledCommands = [ CMD_CONNECT ]
+        self.supportedAuthMechs = [AUTHMECH_ANON]
+        self.supportedAddrs = [ADDR_DOMAINNAME]
+        self.enabledCommands = [CMD_CONNECT]
         self.peersock = None
         self.addressType = 0
         self.requestType = 0
@@ -162,7 +159,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
         try:
             # Parse out data
             ver, nmethod = struct.unpack('!BB', self.buf[:2])
-            methods = struct.unpack('%dB' % nmethod, self.buf[2:nmethod+2])
+            methods = struct.unpack('%dB' % nmethod, self.buf[2:nmethod + 2])
 
             # Ensure version is correct
             if ver != 5:
@@ -171,8 +168,8 @@ class SOCKSv5(protocol.Protocol, FileSender):
                 return
 
             # Trim off front of the buffer
-            self.buf = self.buf[nmethod+2:]
-            
+            self.buf = self.buf[nmethod + 2:]
+
             # Check for supported auth mechs
             for m in self.supportedAuthMechs:
                 if m in methods:
@@ -219,7 +216,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
         result = struct.pack('!BBBBIH', SOCKS5_VER, errorcode, 0, 1, 0, 0)
         self.transport.write(result)
         self.transport.loseConnection()
-    
+
     def _parseRequest(self):
         debug("_parseRequest")
         try:
@@ -235,7 +232,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
             if self.addressType == ADDR_IPV4:
                 addr, port = struct.unpack('!IH', self.buf[4:10])
                 self.buf = self.buf[10:]
-            elif self.addressType == ADDR_DOMAINNAME:            
+            elif self.addressType == ADDR_DOMAINNAME:
                 nlen = ord(self.buf[4])
                 addr, port = struct.unpack('!%dsH' % nlen, self.buf[5:])
                 self.buf = self.buf[7 + len(addr):]
@@ -282,7 +279,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
             if self.addressType == ADDR_IPV4:
                 addr, port = struct.unpack('!IH', self.buf[4:10])
                 self.buf = self.buf[10:]
-            elif self.addressType == ADDR_DOMAINNAME:            
+            elif self.addressType == ADDR_DOMAINNAME:
                 nlen = ord(self.buf[4])
                 addr, port = struct.unpack('!%dsH' % nlen, self.buf[5:])
                 self.buf = self.buf[7 + len(addr):]
@@ -308,7 +305,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
 
     def connectionMade(self):
         debug("connectionMade (mode = %s)" % "requester" if isinstance(self.factory, Socks5ServerFactory) else "target")
-        
+
         if isinstance(self.factory, Socks5ClientFactory):
             self.sid = self.factory.sid
             self.profile = self.factory.profile
@@ -318,9 +315,9 @@ class SOCKSv5(protocol.Protocol, FileSender):
 
     def connectRequested(self, addr, port):
         debug("connectRequested")
-        
+
         # Check that this session is expected
-        if not self.factory.hash_sid_map.has_key(addr):
+        if addr not in self.factory.hash_sid_map:
             #no: we refuse it
             self.sendErrorReply(REPLY_CONN_REFUSED)
             return
@@ -336,7 +333,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
         """Callback called when the result iq is received"""
         d = self.beginFileTransfer(file_obj, self.transport)
         d.addCallback(self.fileTransfered)
-    
+
     def fileTransfered(self, d):
         info(_("File transfer completed, closing connection"))
         self.transport.loseConnection()
@@ -351,10 +348,10 @@ class SOCKSv5(protocol.Protocol, FileSender):
                                  ADDR_DOMAINNAME, len(remotehost), remotehost, remoteport)
         self.transport.write(result)
         self.state = STATE_READY
-    
+
     def bindRequested(self, addr, port):
         pass
-    
+
     def authenticateUserPass(self, user, passwd):
         debug("User/pass: %s/%s", user, passwd)
         return True
@@ -374,14 +371,13 @@ class SOCKSv5(protocol.Protocol, FileSender):
         if self.state == STATE_TARGET_AUTH:
             ver, method = struct.unpack('!BB', buf)
             self.buf = self.buf[2:]
-            if ver!=SOCKS5_VER or method!=AUTHMECH_ANON:
+            if ver != SOCKS5_VER or method != AUTHMECH_ANON:
                 self.transport.loseConnection()
             else:
                 self._makeRequest()
         if self.state == STATE_TARGET_REQUEST:
             self._parseRequestReply()
 
-
     def clientConnectionLost(self, reason):
         debug("clientConnectionLost")
         self.transport.loseConnection()
@@ -390,7 +386,7 @@ class SOCKSv5(protocol.Protocol, FileSender):
         debug("connectionLost")
         if self.state != STATE_CONNECT_PENDING:
             self.transport.unregisterProducer()
-            if self.peersock != None:
+            if self.peersock is not None:
                 self.peersock.peersock = None
                 self.peersock.transport.unregisterProducer()
                 self.peersock = None
@@ -405,10 +401,11 @@ class Socks5ServerFactory(protocol.ServerFactory):
         self.finishedCb = finishedCb
 
     def startedConnecting(self, connector):
-        debug (_("Socks 5 server connection started"))
+        debug(_("Socks 5 server connection started"))
 
     def clientConnectionLost(self, connector, reason):
-        debug (_("Socks 5 server connection lost (reason: %s)"), reason)
+        debug(_("Socks 5 server connection lost (reason: %s)"), reason)
+
 
 class Socks5ClientFactory(protocol.ClientFactory):
     protocol = SOCKSv5
@@ -432,15 +429,15 @@ class Socks5ClientFactory(protocol.ClientFactory):
         self.profile = profile
 
     def startedConnecting(self, connector):
-        debug (_("Socks 5 client connection started"))
+        debug(_("Socks 5 client connection started"))
 
     def clientConnectionLost(self, connector, reason):
-        debug (_("Socks 5 client connection lost (reason: %s)"), reason)
-        self.finishedCb(self.sid, reason.type == jab_error.ConnectionDone, self.profile) #TODO: really check if the state is actually successful
+        debug(_("Socks 5 client connection lost (reason: %s)"), reason)
+        self.finishedCb(self.sid, reason.type == jab_error.ConnectionDone, self.profile)  # TODO: really check if the state is actually successful
 
 
-class XEP_0065():
-    
+class XEP_0065(object):
+
     NAMESPACE = NS_BS
 
     params = """
@@ -463,30 +460,30 @@ class XEP_0065():
 
     def __init__(self, host):
         info(_("Plugin XEP_0065 initialization"))
-        
+
         #session data
-        self.hash_sid_map = {}  #key: hash of the transfer session, value: (session id, profile)
-        
+        self.hash_sid_map = {}  # key: hash of the transfer session, value: (session id, profile)
+
         self.host = host
         debug(_("registering"))
         self.server_factory = Socks5ServerFactory(host, self.hash_sid_map, lambda sid, success, profile: self._killId(sid, success, profile=profile))
 
         #parameters
-        host.memory.importParams(XEP_0065.params)
+        host.memory.updateParams(XEP_0065.params)
         host.memory.setDefault("IP", "File Transfer", self.getExternalIP)
         port = int(self.host.memory.getParamA("Port", "File Transfer"))
-        
+
         info(_("Launching Socks5 Stream server on port %d"), port)
         reactor.listenTCP(port, self.server_factory)
-    
+
     def getHandler(self, profile):
-        return XEP_0065_handler(self)  
-   
+        return XEP_0065_handler(self)
+
     def profileConnected(self, profile):
         client = self.host.getClient(profile)
         if not client:
             raise ProfileNotInCacheError
-        client.xep_0065_current_stream = {} #key: stream_id, value: data(dict)
+        client.xep_0065_current_stream = {}  # key: stream_id, value: data(dict)
 
     def getExternalIP(self):
         """Return IP visible from outside, by asking to a website"""
@@ -503,13 +500,14 @@ class XEP_0065():
             data["size"] = str(client.xep_0065_current_stream[sid]["size"])
         except:
             pass
-    
+
     def _timeOut(self, sid, profile):
         """Delecte current_stream id, called after timeout
         @param id: id of client.xep_0065_current_stream"""
-        info(_("Socks5 Bytestream: TimeOut reached for id %s [%s]") % (sid, profile));
+        info(_("Socks5 Bytestream: TimeOut reached for id %(sid)s [%(profile)s]")
+             % {"sid": sid, "profile": profile})
         self._killId(sid, False, "TIMEOUT", profile)
-    
+
     def _killId(self, sid, success=False, failure_reason="UNKNOWN", profile=None):
         """Delete an current_stream id, clean up associated observers
         @param sid: id of client.xep_0065_current_stream"""
@@ -518,21 +516,21 @@ class XEP_0065():
         if not client:
             warning(_("Client no more in cache"))
             return
-        if not client.xep_0065_current_stream.has_key(sid):
+        if sid not in client.xep_0065_current_stream:
             warning(_("kill id called on a non existant id"))
             return
-        if client.xep_0065_current_stream[sid].has_key("observer_cb"):
+        if "observer_cb" in client.xep_0065_current_stream[sid]:
             xmlstream = client.xep_0065_current_stream[sid]["xmlstream"]
             xmlstream.removeObserver(client.xep_0065_current_stream[sid]["event_data"], client.xep_0065_current_stream[sid]["observer_cb"])
         if client.xep_0065_current_stream[sid]['timer'].active():
             client.xep_0065_current_stream[sid]['timer'].cancel()
-        if client.xep_0065_current_stream[sid].has_key("size"):
+        if "size" in client.xep_0065_current_stream[sid]:
             self.host.removeProgressCB(sid, profile)
-       
+
         file_obj = client.xep_0065_current_stream[sid]['file_obj']
         success_cb = client.xep_0065_current_stream[sid]['success_cb']
         failure_cb = client.xep_0065_current_stream[sid]['failure_cb']
-        
+
         session_hash = client.xep_0065_current_stream[sid].get('hash')
         del client.xep_0065_current_stream[sid]
         if session_hash in self.hash_sid_map:
@@ -544,7 +542,7 @@ class XEP_0065():
         else:
             failure_cb(sid, file_obj, NS_BS, failure_reason, profile)
 
-    def startStream(self, file_obj, to_jid, sid, length, successCb, failureCb, size = None, profile=None):
+    def startStream(self, file_obj, to_jid, sid, length, successCb, failureCb, size=None, profile=None):
         """Launch the stream workflow
         @param file_obj: file_obj to send
         @param to_jid: JID of the recipient
@@ -558,14 +556,14 @@ class XEP_0065():
         if not client:
             error(_("Unknown profile, this should not happen"))
             raise ProfileNotInCacheError
-        
-        if length != None:
+
+        if length is not None:
             error(_('stream length not managed yet'))
-            return;
-        
+            return
+
         profile_jid = client.jid
         xmlstream = client.xmlstream
-        
+
         data = client.xep_0065_current_stream[sid] = {}
         data["timer"] = reactor.callLater(TIMEOUT, self._timeOut, sid, profile)
         data["file_obj"] = file_obj
@@ -579,7 +577,7 @@ class XEP_0065():
         if size:
             data["size"] = size
             self.host.registerProgressCB(sid, self.getProgress, profile)
-        iq_elt = jabber_client.IQ(xmlstream,'set')
+        iq_elt = jabber_client.IQ(xmlstream, 'set')
         iq_elt["from"] = profile_jid.full()
         iq_elt["to"] = to_jid.full()
         query_elt = iq_elt.addElement('query', NS_BS)
@@ -615,10 +613,10 @@ class XEP_0065():
         except KeyError:
             error(_("Internal error, can't do transfer"))
             return
-        
+
         if timer.active():
             timer.cancel()
-        
+
         profile_jid, xmlstream = self.host.getJidNStream(profile)
         query_elt = iq_elt.firstChildElement()
         streamhost_elts = filter(lambda elt: elt.name == 'streamhost-used', query_elt.elements())
@@ -638,7 +636,7 @@ class XEP_0065():
             factory = Socks5ClientFactory(client.xep_0065_current_stream, sid, None, self.activateProxyStream, lambda sid, success, profile: self._killId(sid, success, profile=profile), True, profile)
             reactor.connectTCP(proxy_host, int(proxy_port), factory)
         else:
-            data["start_transfer_cb"](file_obj) #We now activate the stream
+            data["start_transfer_cb"](file_obj)  # We now activate the stream
 
     def activateProxyStream(self, sid, iq_id, start_transfer_cb, profile):
         debug(_("activating stream"))
@@ -648,7 +646,7 @@ class XEP_0065():
         data = client.xep_0065_current_stream[sid]
         profile_jid, xmlstream = self.host.getJidNStream(profile)
 
-        iq_elt = client.IQ(xmlstream,'set')
+        iq_elt = client.IQ(xmlstream, 'set')
         iq_elt["from"] = profile_jid.full()
         iq_elt["to"] = self.host.memory.getParamA("Proxy", "File Transfer", profile_key=profile)
         query_elt = iq_elt.addElement('query', NS_BS)
@@ -686,23 +684,22 @@ class XEP_0065():
         data["timer"] = reactor.callLater(TIMEOUT, self._timeOut, sid, profile)
         data["success_cb"] = success_cb
         data["failure_cb"] = failure_cb
-    
-    
+
     def streamQuery(self, iq_elt, profile):
         """Get file using byte stream"""
         debug(_("BS stream query"))
         client = self.host.getClient(profile)
-        
+
         if not client:
             raise ProfileNotInCacheError
-        
+
         xmlstream = client.xmlstream
 
         iq_elt.handled = True
         query_elt = iq_elt.firstChildElement()
         sid = query_elt.getAttribute("sid")
         streamhost_elts = filter(lambda elt: elt.name == 'streamhost', query_elt.elements())
-        
+
         if not sid in client.xep_0065_current_stream:
             warning(_("Ignoring unexpected BS transfer: %s" % sid))
             self.sendNotAcceptableError(iq_elt['id'], iq_elt['from'], xmlstream)
@@ -717,7 +714,7 @@ class XEP_0065():
             self.sendBadRequestError(iq_elt['id'], iq_elt['from'], xmlstream)
             return
 
-        streamhost_elt = streamhost_elts[0] #TODO: manage several streamhost elements case
+        streamhost_elt = streamhost_elts[0]  # TODO: manage several streamhost elements case
         sh_host = streamhost_elt.getAttribute("host")
         sh_port = streamhost_elt.getAttribute("port")
         sh_jid = streamhost_elt.getAttribute("jid")
@@ -728,10 +725,10 @@ class XEP_0065():
 
         client.xep_0065_current_stream[sid]["streamhost"] = (sh_host, sh_port, sh_jid)
 
-        info (_("Stream proposed: host=[%(host)s] port=[%(port)s]") % {'host':sh_host, 'port':sh_port})
+        info(_("Stream proposed: host=[%(host)s] port=[%(port)s]") % {'host': sh_host, 'port': sh_port})
         factory = Socks5ClientFactory(client.xep_0065_current_stream, sid, iq_elt["id"], self.activateStream, lambda sid, success, profile: self._killId(sid, success, profile=profile), profile=profile)
         reactor.connectTCP(sh_host, int(sh_port), factory)
-                
+
     def activateStream(self, sid, iq_id, profile):
         client = self.host.getClient(profile)
         if not client:
@@ -757,10 +754,10 @@ class XEP_0065():
         result = domish.Element((None, 'iq'))
         result['type'] = 'result'
         result['id'] = iq_id
-        result['to'] = to_jid 
+        result['to'] = to_jid
         error_el = result.addElement('error')
         error_el['type'] = 'modify'
-        error_el.addElement(('urn:ietf:params:xml:ns:xmpp-stanzas','not-acceptable'))
+        error_el.addElement(('urn:ietf:params:xml:ns:xmpp-stanzas', 'not-acceptable'))
         xmlstream.send(result)
 
     def sendBadRequestError(self, iq_id, to_jid, xmlstream):
@@ -771,15 +768,16 @@ class XEP_0065():
         result = domish.Element((None, 'iq'))
         result['type'] = 'result'
         result['id'] = iq_id
-        result['to'] = to_jid 
+        result['to'] = to_jid
         error_el = result.addElement('error')
         error_el['type'] = 'cancel'
-        error_el.addElement(('urn:ietf:params:xml:ns:xmpp-stanzas','bad-request'))
+        error_el.addElement(('urn:ietf:params:xml:ns:xmpp-stanzas', 'bad-request'))
         xmlstream.send(result)
 
+
 class XEP_0065_handler(XMPPHandler):
     implements(iwokkel.IDisco)
-    
+
     def __init__(self, plugin_parent):
         self.plugin_parent = plugin_parent
         self.host = plugin_parent.host
@@ -801,30 +799,29 @@ class XEP_0065_handler(XMPPHandler):
         if len(streamhost_elts) != 1:
             warning(_("Multiple streamhost elements in proxy not managed, keeping only the first one"))
         streamhost_elt = streamhost_elts[0]
-        proxy = self.host.memory.setParam("Proxy", streamhost_elt.getAttribute("jid",""), "File Transfer", self.parent.profile)
-        proxy = self.host.memory.setParam("Proxy host", streamhost_elt.getAttribute("host",""), "File Transfer", self.parent.profile)
-        proxy = self.host.memory.setParam("Proxy port", streamhost_elt.getAttribute("port",""), "File Transfer", self.parent.profile)
+        proxy = self.host.memory.setParam("Proxy", streamhost_elt.getAttribute("jid", ""),
+                                          "File Transfer", profile_key=self.parent.profile)
+        proxy = self.host.memory.setParam("Proxy host", streamhost_elt.getAttribute("host", ""),
+                                          "File Transfer", profile_key=self.parent.profile)
+        proxy = self.host.memory.setParam("Proxy port", streamhost_elt.getAttribute("port", ""),
+                                          "File Transfer", profile_key=self.parent.profile)
 
-    
     def connectionInitialized(self):
         def after_init(ignore):
-            proxy_ent = self.host.memory.getServerServiceEntity("proxy", "bytestreams", self.parent.profile)
+            proxy_ent = self.host.memory.getServerServiceEntity("proxy", "bytestreams", profile=self.parent.profile)
             if not proxy_ent:
                 debug(_("No proxy found on this server"))
                 return
-            iq_elt = jabber_client.IQ(self.parent.xmlstream,'get')
+            iq_elt = jabber_client.IQ(self.parent.xmlstream, 'get')
             iq_elt["to"] = proxy_ent.full()
             query_elt = iq_elt.addElement('query', NS_BS)
             iq_elt.addCallback(self._proxyDataResult)
             iq_elt.send()
 
-
-        self.xmlstream.addObserver(BS_REQUEST, self.plugin_parent.streamQuery, profile = self.parent.profile)
-        proxy = self.host.memory.getParamA("Proxy", "File Transfer", profile_key = self.parent.profile)
+        self.xmlstream.addObserver(BS_REQUEST, self.plugin_parent.streamQuery, profile=self.parent.profile)
+        proxy = self.host.memory.getParamA("Proxy", "File Transfer", profile_key=self.parent.profile)
         if not proxy:
             self.parent.client_initialized.addCallback(after_init)
-            
-
 
     def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
         return [disco.DiscoFeature(NS_BS)]
